#
# Awk program that takes as input a C++ header file and outputs
# a UNIX style man page.
#
# The contents of this file are public domain.
# Author: Bob Mastors
#         Epoch Systems, Inc.
#         6/90
#	  Version 2

BEGIN {
    library= "UNKNOWN"
    sourcefilecount= 1
    hfilecount= 1
    definecount= 1
    constcount= 1
    includecount= 1
    typedefcount= 1
    enumcount= 1
    structcount= 1
    externcount= 0
    classcount= 0
    examplecount= 1
    doccount= 0
    lp_indent= "    "
    bold= ""

    define[0]=      "DEFINES"
    const[0]=       "CONSTANTS"
    include[0]=     "INCLUDES"
    typedef[0]=     "TYPE DEFINITIONS"
    enum[0]=        "ENUMERATIONS"
    hdr_printed = 0;
    struct[0]=      "STRUCTURES"     
    mifconcat = "'>\n      <Char HardReturn >>\n    <ParaLine\n      <String `"
    }

/^#define/ {
    tmp= substr( $0, 8 )
    tmp= strip_white( tmp )
    gsub( "\\\\$", "", tmp );
    if (definecount != 1)
      {
      if ( index (tmp, "__") != 1) define[ definecount++ ] = "'><Char HardReturn>>\n  <ParaLine <String `" tmp;
      }
    else
      {
      if ( index (tmp, "__") != 1) define[ definecount++ ] = tmp;
      }
    }

/^const/ {
    tmp= substr( $0, 6 )
    tmp= strip_white( tmp )
    gsub( "//", "", tmp );
    const[ constcount++ ] = tmp
    }

/^#include/ {
    tmp= substr( $0, 9 )
    tmp= strip_white( tmp )
    include[ includecount++ ] = tmp
    }

/^extern/ {
    tmp= substr( $0, 7 )
    tmp= strip_white( tmp )
    extern[ externcount++ ] = tmp
    }

/^typedef/ {
    tmp= substr( $0, 8 )
    tmp= strip_white( tmp )
    typedef[ typedefcount++ ] = tmp
    }

/^enum/ {
    tmp= substr( $0, 5 )
    gsub( "{", "", tmp );
    tmp= strip_white( tmp )
    enum[ enumcount++ ] = tmp
    while( (getline tmp) > 0 )
        {
        gsub( "{", "", tmp );
        gsub( ",", " ", tmp );
        gsub( "//", "", tmp );
        tmp= strip_white( tmp )
        if( tmp ~ "} *;" )
            break;
	if( length(tmp) > 0 )
	    enum[ enumcount++ ] = "%@" tmp
        }
    }

/^struct/ {
    process_struct( $0 )
    }

/^class/ {
    # don't get class declarations, only definitions
    pos= match( $0, "[;]" );
    if ( pos == 0 )
      {
      process_class( $0 )
      classcount++
      }
    }

/^\/\// {
    tmp= substr( $0, 4 )
    striptmp= strip_white( tmp )
    if( gsub( "^\.EXAMPLE", "", striptmp ) == 1 )
        {
        examples[examplecount++]= strip_white( striptmp )
        }
    else
    if( gsub( "^\.NAME", "", striptmp ) == 1 )
        {
        namehead= strip_white( striptmp )
        }
    else
    if( gsub( "^\.HEADER", "", striptmp ) == 1 )
        {
        striptmp= strip_white( striptmp )
	header= toupper( striptmp )
        }
    else
    if( gsub( "^\.INCLUDE", "", striptmp ) == 1 )
        {
        includefile= strip_white( striptmp )
        }
    else
    if( gsub( "^\.SECTION", "", striptmp ) == 1 )
        {
        striptmp= strip_white( striptmp )
        striptmp= toupper( striptmp )
        doc[ doccount++ ]= ".SECTION"
        doc[ doccount++ ]= striptmp
        doc_flag= "true"
        }
    else if( doc_flag == "true" )
        {
        gsub( "[\t ]*$", "", tmp);
	gsub( "'", "\\xd5 ", tmp);
	gsub( ">", "\\>", tmp);
        doc[ doccount++ ]= tmp
        }
    }

NF == 0 {
    if( doc_flag == "true" )
	{
        doc_flag= ""
	}
    }

END {
    # get the modify date of the file
    if( length(mtime) == 0 )
	{
	"ls -l " FILENAME | getline tmp
	date= substr( tmp, 42, 12 );
	}
    else
	{
	mtime " " FILENAME | getline date
	}

    # If .NAME not specified, then make one up
    if( length( namehead ) == 0 )
	namehead=  FILENAME " - ???"

    # Make a name for the man page from the first word in the
    # NAME section.
    pos= match( namehead, "[ ,]" );
    if( pos == 0 )
	titlename= namehead
    else
	titlename= substr( namehead, 1, pos-1 )

    # If .HEADER not specified then use C++
    if( length( header ) == 0 )
        header= "vtk"

    # If .LIBRARY not specified then use c++
    if( length( library ) == 0 )
        library= "vtk"

    # Start the formatting and print the title and name
    format_start( titlename, library, header, date, namehead );

    # print the synopsis header
    print "  <Para <PgfTag `section'> <ParaLine <String `CLASS HIERARCHY'>>>";
    
    output_class()
    output_section( extern )
    output_section( doc )

    if( examplecount > 1) output_examples()

#    if( structcount > 1 ) output_section( struct )
#    if( enumcount > 1 ) output_section( enum )
#    if( typedefcount > 1 ) output_section( typedef )
    if( constcount > 1 ) output_section( const )
    if( definecount > 1 ) output_section( define )
#    if( includecount > 1 ) output_section( include )
    output_section( sourcefile )

    output_summarys_from_h(hfile)
    if ( length(sourcefile) > 0 ) output_summarys(sourcefile)

    # end the formatting
    format_end()
    }

function expand_macro (input, trailer)
{
    if (index(input,"SetMacro") != 0) {
            start_name=index(input,"(");
            end_name=index(input,",");
            name= substr( input, start_name + 1, end_name - start_name - 1);
            start_type=end_name + 1;
            end_type=index(substr( input, start_type), ")");
            type= substr(input, start_type, end_type - 1);
            output= "void Set" name "(" type ")" trailer ;
    }
    else

    if (index(input,"SetClampMacro") != 0) {
            start_name=index(input,"(");
            end_name=index(input,",");
            name= substr( input, start_name + 1, end_name - start_name - 1);
            start_type=end_name + 1;
            end_type=index(substr( input, start_type), ",");
            type= substr(input, start_type, end_type - 1);
            output= "void Set" name "(" type ")" trailer ;
    }
    else

    if (index(input,"GetMacro") != 0) {
            start_name=index(input,"(");
            end_name=index(input,",");
            name= substr( input, start_name + 1, end_name - start_name - 1);
            start_type=end_name + 1;
            end_type=index(substr( input, start_type), ")");
            type= substr(input, start_type, end_type - 1);
            output= type " Get" name "()" trailer ;
    }
    else

    if (index(input,"SetStringMacro") != 0) {
            start_name=index(input,"(");
            end_name=index(input,")");
            name= substr( input, start_name + 1, end_name - start_name - 1);
            output= "void Set" name "(char *)" trailer ;
    }
    else

    if (index(input,"GetStringMacro") != 0) {
            start_name=index(input,"(");
            end_name=index(input,")");
            name= substr( input, start_name + 1, end_name - start_name - 1);
            output= "char *Get" name "()" trailer ;
    }
    else

    if (index(input,"SetObjectMacro") != 0) {
            start_name=index(input,"(");
            end_name=index(input,",");
            name= substr( input, start_name + 1, end_name - start_name - 1);
            start_type=end_name + 1;
            end_type=index(substr( input, start_type), ")");
            type= substr(input, start_type, end_type - 1);
            output= "void Set" name "(" type "*)" trailer mifconcat "void '><Char Tab><String `Set" name "(" type "& )" trailer ;;
    }
    else

    if (index(input,"SetRefCountedObjectMacro") != 0) {
            start_name=index(input,"(");
            end_name=index(input,",");
            name= substr( input, start_name + 1, end_name - start_name - 1);
            start_type=end_name + 1;
            end_type=index(substr( input, start_type), ")");
            type= substr(input, start_type, end_type - 1);
            output= "void Set" name "(" type "*)" trailer mifconcat "void '><Char Tab><String `Set" name "(" type "& )" trailer ;;
    }
    else

    if (index(input,"GetObjectMacro") != 0) {
            start_name=index(input,"(");
            end_name=index(input,",");
            name= substr( input, start_name + 1, end_name - start_name - 1);
            start_type=end_name + 1;
            end_type=index(substr( input, start_type), ")");
            type= substr(input, start_type, end_type - 1);
            output= type " *Get" name "()" trailer ;
    }
    else

    if (index(input,"BooleanMacro") != 0) {
            start_name=index(input,"(");
            end_name=index(input,",");
            name= substr( input, start_name + 1, end_name - start_name - 1);
            start_type=end_name + 1;
            end_type=index(substr( input, start_type), ")");
            type= substr(input, start_type, end_type - 1);
            output= "void " name "On()" trailer mifconcat "void '><Char Tab><String `" name "Off()" trailer ;
    }
    else

    if (index(input,"SetVector2Macro") != 0) {
            start_name=index(input,"(");
            end_name=index(input,",");
            name= substr( input, start_name + 1, end_name - start_name - 1);
            start_type=end_name + 1;
            end_type=index(substr( input, start_type), ")");
            type= substr(input, start_type, end_type - 1);
            output= "void Set" name "(" type ", " type ")" trailer mifconcat "void '><Char Tab><String `Set" name "(" type " *)" trailer ;
    }
    else

    if (index(input,"SetVector3Macro") != 0) {
            start_name=index(input,"(");
            end_name=index(input,",");
            name= substr( input, start_name + 1, end_name - start_name - 1);
            start_type=end_name + 1;
            end_type=index(substr( input, start_type), ")");
            type= substr(input, start_type, end_type - 1);
            output= "void Set" name "(" type ", " type ", " type ")" trailer mifconcat "void '><Char Tab><String `Set" name "(" type " *)" trailer ;
    }
    else

    if (index(input,"SetVector4Macro") != 0) {
            start_name=index(input,"(");
            end_name=index(input,",");
            name= substr( input, start_name + 1, end_name - start_name - 1);
            start_type=end_name + 1;
            end_type=index(substr( input, start_type), ")");
            type= substr(input, start_type, end_type - 1);
            output= "void Set" name "(" type ", " type ", " type ", " type ")" trailer mifconcat "void '><Char Tab><String `Set" name "(" type " *)" trailer ;
    }
    else

    if (index(input,"SetVectorMacro") != 0) {
            start_name=index(input,"(");
            end_name=index(input,",");
            name= substr( input, start_name + 1, end_name - start_name - 1);
            start_type=end_name + 1;
            end_type=index(substr( input, start_type), ",");
            type= substr(input, start_type, end_type - 1);
            start_count=end_name+end_type + 1;
            end_count=index(substr( input, start_count), ")");
            count= substr(input, start_count, end_count - 1);
            output= "void Set" name "(" type " data[" count "])" trailer mifconcat "void '><Char Tab><String `Set" name "(" type " *)" trailer ;
    }
    else

    if (index(input,"GetVectorMacro") != 0) {
            start_name=index(input,"(");
            end_name=index(input,",");
            name= substr( input, start_name + 1, end_name - start_name - 1);
            start_type=end_name + 1;
            end_type=index(substr( input, start_type), ",");
            type= substr(input, start_type, end_type - 1);
            start_count=end_name+end_type + 1;
            end_count=index(substr( input, start_count), ")");
            count= substr(input, start_count, end_count - 1);
            output= type " *Get" name "()" trailer mifconcat "void '><Char Tab><String `Get" name "(" type " data[" count "])" trailer;
    }

    return output;
}

# Call to process a class.  Pass the name of the class.
function process_class( name,    inputstr,ii,braceLevel,idx,tmpStr)
    {
    privatecount= 1
    publiccount= 1
    protectedcount= 1
    braceLevel = 1

    # We do not record the curly braces surrounding the class
    # but we record all others.

    # If an open curly brace on the "class" line then incriment the
    # indent level.
    # Also see if k&r style curly brace alignment.
    if( name ~ ".*{.*" )
	{
	style= "k&r"
	sub( "{", "", name );
	}
    else
	style= "custom"

    # record the name
    name= strip_white( name )
    class[ classcount, "head", 0 ]= name

    # get lines of input until EOF
    ii= "private"
    while( (getline inputstr) > 0 )
        {
        # just get the token(s), ignore blank lines
        inputstr= strip_white( inputstr )
	if( length(inputstr) == 0 )
	    continue;

        # check on curly brace level
        tmpStr = inputstr
        idx = index(tmpStr, "{");
        while  ( idx != 0 )
           {
           braceLevel++
           tmpStr = substr(tmpStr,idx+1)
           idx = index(tmpStr, "{");
           }
        tmpStr = inputstr
        idx = index(tmpStr, "}");
        while  ( idx != 0 )
           {
           braceLevel--
           tmpStr = substr(tmpStr,idx+1)
           idx = index(tmpStr, "}");
           }

        # see if should adjust the indent level
	if( style == "k&r" )
	    {
	    if( inputstr ~ "}.*" )
	        {
	        }
	    }
	else
	    {
	    if( inputstr ~ ".*{.*" )
	        {
	        }
	    else
	    if( inputstr ~ "}.*" )
	        {
		}
	    }

        # termination of class
        if( braceLevel == 1 )
            ii= "alldone"

        # public members follow
        else if( inputstr ~ "public *:" )
            ii= "public"

        # private members follow
        else if( inputstr ~ "private *:" )
            ii= "private"

        # protected members follow
        else if( inputstr  ~ "protected *:" )
            ii= "protected"

        else if( inputstr  ~ "^[ \t]*//")
            continue;

        else
            {
            gsub( "//", "  ", inputstr )
	    tmp= inputstr
	    # convert macros to method names
            if( index(inputstr, "Macro") != 0)
              {
              tmp = expand_macro(tmp, ";");
              }

	    if( ii == "private" )
		class[ classcount, "private", privatecount++ ]= tmp
	    else
	    if( ii == "public" )
		class[ classcount, "public", publiccount++ ]= tmp
	    else
	    if( ii == "protected" )
		class[ classcount, "protected", protectedcount++ ]= tmp
	
            }

        # see if should adjust the indent level
	if( style == "k&r" )
	    {
	    if( inputstr ~ ".*{.*" )
	        {
	        }
	    }
	else
	    {
	    if( inputstr ~ "}.*" )
	        {
	        }
	    }
	}

    class[ classcount, "public", 0 ]= publiccount
    class[ classcount, "protected", 0 ]= protectedcount
    class[ classcount, "private", 0 ]= privatecount
    }

# processes a structure
function process_struct( tmp,     style,indent_level )
    {
    indent_level = ""

    if( tmp ~ ".*{.*" )
	style= "k&r"
    else
	style= "custom"

    do
        {
        tmp= strip_white( tmp )
        gsub( "//", "  ", tmp );

	if( style == "k&r" )
	    {
	    if( tmp ~ "}.*" )
	        {
	        indent_level= substr( indent_level, 3 )
	        if( length(indent_level) == 0 )
		    {
		    struct[ structcount++ ] = indent_level tmp
                    break;
		    }
	        }
	    struct[ structcount++ ] = indent_level tmp
	    if( tmp ~ ".*{.*" )
	        {
	        }
	    }
	else
	    {
	    if( tmp ~ ".*{.*" )
	        {
	        }
	    struct[ structcount++ ] = indent_level tmp
	    if( tmp ~ "}.*" )
	        {
	        indent_level= substr( indent_level, 3 )
	        if( length(indent_level) == 0 )
                    break;
	        }
	    }

        }while( (getline tmp) > 0 )
    }

#
# Outputs the specified array.
#
function output_section( data,   tmp,xx )
{
# Traverse the list of lines.
  xx= 0;
  firstline = 1;

  while( xx in data )
    {
    # if we find the .SECTION keyword then we have a new section
    if (index(data[xx],".SECTION") != 0)
      {
      firstline = 1;
      if (xx != 0)
	{
	print "  >";
	}
      xx++;
      print  "  <Para <PgfTag `section'> <ParaLine";
      printf("    <String `%s'>>>\n",data[xx]);
      print  "  <Para <PgfTag `block pgf'>";
      }
    else
      {
      # if this is the very first entry then it must be a section name
      if (xx == 0)
	{
	print  "  <Para <PgfTag `section'> <ParaLine";
	printf("    <String `%s'>>>\n",data[xx]);
	print  "  <Para <PgfTag `block pgf'>";
	}
      else
	{
        # is this a blank line do a CR
        if (length(data[xx]) == 0)
          {
          print  "    <ParaLine <Char HardReturn>>";
          print  "    <ParaLine <Char HardReturn>>";
	  firstline = 1;
          }
        else
          {
          # if not the first line then remember to add a leading space
          if (firstline == 1)
            {
	    printf("    <ParaLine <String `%s'>>\n",data[xx]);
            firstline = 0;
            }
          else
            {
	    printf("    <ParaLine <String ` %s'>>\n",data[xx]);
            }
          }
	}
      }
    xx++;
    }

  # if there was at least one real line then close the paragraph
  if (xx != 0)
    {
    print "  >";
     }
}

# Special funtion for printing out classes because the class
# was built as a multi-dimensional array.
function output_class(   xx,hd,cname)
    {
    # for each class
    for( xx= 0; xx < classcount; xx++ )
	{
	# output the name of the class
	print  "  <Para <PgfTag `block pgf'> <ParaLine ";
	print  "    <Font <FTag `'> <FWeight `Bold'> ";
	print  "      <FPostScriptName `Times-Bold'>> ";
	printf("    <String `%s'>>\n",class[ xx, "head", 0]);
        print  "    <ParaLine <Font <FTag `'>>>>";

#	output_part_class( "" "", "public", xx );
#	output_part_class( "%@" "Protected members", "protected", xx );
#	output_part_class( "%@" "Private members", "private", xx );
	}
    }

# Prints the specified portion of the class.
function output_part_class( header, idname, idnum,     yy,amount )
  {
  # get the number of items
  amount= class[ idnum, idname, 0 ];
  if( amount == 1 )
    return;

  print  "  <Para <PgfTag `block pgf'>";
  print  "    <ParaLine <Font <FTag `'> <FWeight `Bold'>";
  print  "      <FPostScriptName `Times-Bold'>>";
  printf("      <String `%s'>\n",class[idnum,idname, 1]);
  print  "      <Char HardReturn >>";

  for( yy= 2; yy < amount; yy++ )
    {
    print  "    <ParaLine ";
    printf("      <String `%s'>\n",class[idnum,idname, yy]);
    print  "      <Char HardReturn >>";
    }

  print  "    <ParaLine <Font <FTag `'>>>>";
}

function output_one_summary()
{
  if (!hdr_printed)
    {
    # print the header
    print "  <Para <PgfTag `section'> <ParaLine <String `SUMMARY'>>>";
    hdr_printed = 1;
    }

# output what we collected for the description
  print  "  <Para <PgfTag `section2'>";
  for (yy = 0; yy < titlecount; yy++)
    {
    if (yy != 0)
      {
      print  "      <Char HardReturn>>";
      }
    printf("    <ParaLine <String `%s'>\n",title[yy]);
    }
  print  "  >>";

  print  "  <Para <PgfTag `block pgf3'>";
  for( yy= 0; yy < datacount; yy++ )
    {
    print  "    <ParaLine";
    if (yy == 0)
      {
      printf("    <String `%s'>>\n",data[yy]);
      }
    else
      {
      printf("    <String ` %s'>>\n",data[yy]);
      }
    }
  print  "  >";
}


# Call to get the function summarys from the hh files and print them
function output_summarys_from_h(sumfile)
{
  # number of descriptions found
  descriptioncount= 0;

  # loop reading each line of the source file
  while( (getline inputstr <sumfile) > 0 )
    {
    inputstr = strip_white(inputstr);
    # see if the start of the function summary
    if( inputstr == "// Description:" )
      {
      if( descriptioncount > 0 )
	descriptioncount++;
      
      # number of lines to output for this description
      datacount= 0;
      titlecount= 0;

      # loop reading the summary
      while( (getline inputstr <sumfile) > 0 )
        {
	
        inputstr = strip_white(inputstr);
        # see if all done with this description
        if( index(inputstr, "virtual") != 0)
          {
          # output what we collected for the description
	  # remove anything in braces
	  gsub( "{(.)*$", "", inputstr);
	  sub( " " , " '><Char Tab><String `", inputstr);
	  title[ titlecount++ ] = inputstr;
          }
        else
	  if( index(inputstr, "Macro") != 0)
	    {
            # output what we collected for the description
	    title = expand_macro(inputstr, "");
	    sub( " " , " '><Char Tab><String `", title);
	    title[ titlecount++ ] = title;
	    }
	  else
	    if( index(inputstr, "inline") != 0)
	      {
              # output what we collected for the description
	      # inline methods we stop right here, no more lines
	      sub( "inline ", "", inputstr);

	      sub( "vtk[A-Za-z0-9]*::", "", inputstr);
	      
	      bracketPos= match( inputstr, "[{]" );
	      if ( bracketPos > 0 )
		{
		inputstr= substr(inputstr, 1, bracketPos-1);
		}
              # output what we collected for the description
	      sub( " " , " '><Char Tab><String `", inputstr);
	      title[ titlecount++ ] = inputstr;
	      output_one_summary();
	      break;
	      }
	    else
              # strip the comment and leading whitespace
	      if( gsub( "//[\t ]*", "", inputstr ) != 0 )
	        {
                # only keep lines not ending in :
	        if( inputstr !~ ".*:$" )
	          {
                  # save the line of data after removing trailing spaces
                  gsub( "[\t ]*$", "", inputstr );
                  gsub( "'", "\\xd5 ", inputstr );
                  gsub( ">", "\\>", inputstr );
	          data[ datacount++ ] = inputstr;
	          }
	        }
	      else
	        {
                # only keep lines that are not blank
	        if( length( inputstr ) )
	          {
                  # save the line of the function name
                  # remove anything in braces
		  gsub( "{(.)*$", "", inputstr);
	          sub( " " , " '><Char Tab><String `", inputstr);
	          title[ titlecount++ ] = inputstr;
	          }
	        # if we get to a blank line then we should be done
	        # try to get the hell out of here
	        else
	          {
	          output_one_summary();
                  break;
	          }
	        }
	}
      }
    }
}

# Call to get the function summarys from the source files and print them
function output_summarys(sumfile)
{
  # just return if no source files listed
  # number of descriptions found
  descriptioncount= 0;

  # walk the list of source files
  # loop reading each line of the source file
  while( (getline inputstr <sumfile) > 0 )
    {
    inputstr = strip_white(inputstr);
    # see if the start of the function summary
    if( inputstr == "// Description:" )
      {
      descriptioncount++;
	
      # number of lines to output for this description
      datacount= 0;
      titlecount= 0;
      
      # loop reading the summary
      while( (getline inputstr <sumfile) > 0 )
	{
        # see if all done with this description
	if( (inputstr ~ "(^[\t ]*[:{].*)"))
	  {
          # output what we collected for the description
          output_one_summary();
	  break;
	  }

        # strip the comment and leading whitespace
	if( gsub( "//[\t ]*", "", inputstr ) != 0 )
	  {
          # only keep lines not ending in :
	  if( inputstr !~ ".*:$" )
	    {
            # save the line of data after removing trailing spaces
            gsub( "[\t ]*$", "", inputstr );
	    gsub( "'", "\\xd5 ", inputstr );
	    gsub( ">", "\\>", inputstr );
	    data[ datacount++ ] = inputstr;
	    }
	  }
	else
	  {
          # only keep lines that are not blank
	  if( length( inputstr ) )
	    {
            # save the line of the function name
	    # remove anything in braces
	    gsub( "{(.)*$", "", inputstr);
	    sub( "vtk[A-Za-z0-9]*::", "", inputstr);
	    sub( " " , " '><Char Tab><String `", inputstr);
	    title[ titlecount++ ] = inputstr;
	    }
	  }
	}
      }
    }
}


# strips leading and trailing whitespace from the specified string and
# returns the string
function strip_white( data )
{
  gsub( "^[\t ]+", "", data );
  gsub( "[\t ]+$", "", data );
  return data;
}

# This routine is called once to perform any device specific initialization
# and to setup the title line and the date line.
function format_start( titlename, library, header, date, namehead )
  {
  print  "#<MIFFile 4.00>";
  print  "# Generated automatically by genmif.awk version 1 from ";
  print  "#     "    FILENAME;
  print  "#";
  print  "<TextFlow <TFTag `A'> <TFAutoConnect Yes > ";
  print  "  <Para <PgfTag `manobjp'> <Pgf <PgfPlacement PageTop >> <ParaLine";
  printf("    <String `%s'>>>\n",titlename);
  print  "  <Para <PgfTag `section'> <ParaLine <String `NAME'>>>";
  print  "  <Para <PgfTag `block pgf'> <ParaLine ";
  printf("    <String `%s'>>>\n",namehead);
}


# This routine is called once to perform any device specific
# cleanup.
function format_end()
{
  print ">";
}

function output_examples ()
{
  print  "  <Para <PgfTag `section'> <ParaLine";
  print  "    <String `EXAMPLES'>>>";
  print  "    <Para <PgfTag `block pgf'> <ParaLine";

  for (xxx = 1; xxx < examplecount; xxx++) 
    {
    if (xxx != 1)
      {
      printf("      <String `, %s'>>\n",examples[xxx]);
      }
    else
      {
      printf("      <String `%s'>>\n",examples[xxx]);
      }
    }
  print  ">";
}







