# Converts gradient vectors and magnitude into a structured points with
# scalars and vectors.  It displays them with hedge hogs.
source vtkInt.tcl

# First create the render master
vtkRenderMaster rm;

# Now create the RenderWindow and Renderer.
#
set renWin [rm MakeRenderWindow];
set ren1   [$renWin MakeRenderer];
set iren [$renWin MakeRenderWindowInteractor];

## Create an image pipeline
vtkImageShortReader reader;
#reader SetDimensions 20 20 20;
#reader SetFilePrefix "test"
#reader SetPixelMask 0xffff;
reader SwapBytesOn;
reader SetDimensions 64 64 93;
reader SetFilePrefix "/home/martink/vtk/data/headsq/quarter"
reader SetAspectRatio 3.2 3.2 1.5 0
reader SetOutputScalarType 1;

vtkImageGaussianSmooth smooth;
smooth SetDimensionality 3;
smooth SetInput [reader GetOutput];
smooth SetStandardDeviation 1.5;
smooth SetRadiusFactor 2.0;

vtkImageGradient gradient;
gradient SetInput [smooth GetOutput];
gradient SetAxes 0 1 2;
gradient ReleaseDataFlagOff;

vtkImageMagnitude magnitude;
magnitude SetInput [gradient GetOutput];

vtkImageNonMaximumSuppression suppress;
suppress SetVectorInput [gradient GetOutput];
suppress SetMagnitudeInput [magnitude GetOutput];
suppress SetAxes 0 1 2;
suppress ReleaseDataFlagOff;
 
vtkImageToStructuredPoints image;
image SetScalarInput [suppress GetOutput];
image SetVectorInput [gradient GetOutput];
image SetAxes 0 1 2; 

vtkImageToStructuredPoints image2;
image2 SetScalarInput [magnitude GetOutput];
image2 SetVectorInput [gradient GetOutput];
image2 SetAxes 0 1 2; 

# create vis pipeline
#
vtkHedgeHog hhog;
    hhog SetInput [image2 GetOutput];
    hhog SetScaleFactor 0.005;
    hhog Update;
vtkPolyMapper hhogMapper;
    hhogMapper SetInput [hhog GetOutput];
    eval hhogMapper SetScalarRange [[hhog GetOutput] GetScalarRange];
vtkActor hhogActor;
    hhogActor SetMapper hhogMapper;

vtkOutlineFilter outline;
    outline SetInput [image GetOutput];
vtkPolyMapper outlineMapper;
    outlineMapper SetInput [outline GetOutput];
vtkActor outlineActor;
    outlineActor SetMapper outlineMapper;

vtkLinkSurfels linker;
linker SetGradientThreshold 40;
linker SetInput [image GetOutput];
linker Update;

vtkSubPixelPositionEdgels spp;
spp SetInput [linker GetOutput];
spp SetGradMaps [image2 GetOutput];

#vtkPolyConnectivityFilter pcf;
#pcf SetInput [spp GetOutput];
#pcf SetScalarRange 20 100000;
#pcf ScalarConnectivityOn;

vtkPolyNormals pn;
pn SetInput [spp GetOutput];
pn SetFeatureAngle 90;

vtkPolyMapper surfMapper;
    surfMapper SetInput [pn GetOutput];
vtkActor surfActor;
    surfActor SetMapper surfMapper;


# Add the actors to the renderer, set the background and size
#
$ren1 AddActors outlineActor;
#$ren1 AddActors hhogActor;
$ren1 AddActors surfActor;
$ren1 SetBackground 1 1 1;
$renWin SetSize 500 500;
$ren1 SetBackground 0.1 0.2 0.4;
$iren Initialize;

# render the image
#
$iren SetUserMethod {wm deiconify .vtkInteract};
[$ren1 GetActiveCamera] Zoom 1.4;
$renWin Render;

$renWin SetFilename tst1.ppm

# prevent the tk window from showing up then start the event loop
wm withdraw .


