/*=========================================================================

  Program:   Visualization Toolkit
  Module:    $RCSfile: vtkImageMedian.h,v $
  Language:  C++
  Date:      $Date: 1996/10/09 15:20:03 $
  Version:   $Revision: 1.10 $
  Thanks:    Thanks to C. Charles Law who developed this class.

Copyright (c) 1993-1995 Ken Martin, Will Schroeder, Bill Lorensen.

This software is copyrighted by Ken Martin, Will Schroeder and Bill Lorensen.
The following terms apply to all files associated with the software unless
explicitly disclaimed in individual files. This copyright specifically does
not apply to the related textbook "The Visualization Toolkit" ISBN
013199837-4 published by Prentice Hall which is covered by its own copyright.

The authors hereby grant permission to use, copy, and distribute this
software and its documentation for any purpose, provided that existing
copyright notices are retained in all copies and that this notice is included
verbatim in any distributions. Additionally, the authors grant permission to
modify this software and its documentation for any purpose, provided that
such modifications are not distributed without the explicit consent of the
authors and that existing copyright notices are retained in all copies. Some
of the algorithms implemented by this software are patented, observe all
applicable patent law.

IN NO EVENT SHALL THE AUTHORS OR DISTRIBUTORS BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT
OF THE USE OF THIS SOFTWARE, ITS DOCUMENTATION, OR ANY DERIVATIVES THEREOF,
EVEN IF THE AUTHORS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

THE AUTHORS AND DISTRIBUTORS SPECIFICALLY DISCLAIM ANY WARRANTIES, INCLUDING,
BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
PARTICULAR PURPOSE, AND NON-INFRINGEMENT.  THIS SOFTWARE IS PROVIDED ON AN
"AS IS" BASIS, AND THE AUTHORS AND DISTRIBUTORS HAVE NO OBLIGATION TO PROVIDE
MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.


=========================================================================*/
// .NAME vtkImageMedian - Median Filter
// .SECTION Description
// vtkImageMedian a Median filter that replaces each pixel with the 
// median value from a square neighborhood around that pixel.
// Neighborhoods can be no more than 3 dimensional.


#ifndef __vtkImageMedian_h
#define __vtkImageMedian_h


#include "vtkImageSpatialFilter.h"

class vtkImageMedian : public vtkImageSpatialFilter
{
public:
  vtkImageMedian();
  ~vtkImageMedian();
  char *GetClassName() {return "vtkImageMedian";};

  // Set/Get the size of the neighood.
  void SetKernelSize(int num, int *size);
  vtkImageSetMacro(KernelSize,int);
  
  void ClearMedian();
  void AccumulateMedian(double val);
  double GetMedian();
  
protected:
  // stuff for sorting the pixels
  int NumNeighborhood;
  double *Sort;
  double *Median;
  int UpMax;
  int DownMax;
  int UpNum;
  int DownNum;

  void Execute(vtkImageRegion *inRegion, vtkImageRegion *outRegion);

};

#endif



